/************************************************************************

	Descriptors.c

	Nintendo Four Score USB Joystick Adapter
	Copyright (C) 2013 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#include "Descriptors.h"

// HID class report descriptor

// Joystick 0 report (note: has 3 extra buttons in order to report the four
// score button states as well as joystick 0)
const USB_Descriptor_HIDReport_Datatype_t PROGMEM JoystickReport0[] =
{
	HID_RI_USAGE_PAGE(8, 0x01),		// Generic Desktop
	HID_RI_USAGE(8, 0x04),			// Joystick
	HID_RI_COLLECTION(8, 0x01),		// Application

	HID_RI_COLLECTION(8, 0x00),		// Physical
	
	// X, Y and throttle
	HID_RI_USAGE(8, 0x30), // Usage X
	HID_RI_USAGE(8, 0x31), // Usage Y
	HID_RI_LOGICAL_MINIMUM(8, -127),
	HID_RI_LOGICAL_MAXIMUM(8, 127),
	HID_RI_PHYSICAL_MINIMUM(8, -1),
	HID_RI_PHYSICAL_MAXIMUM(8, 1),
	HID_RI_REPORT_COUNT(8, 0x02),  // 2 usage reports x and y
	HID_RI_REPORT_SIZE(8, 0x08),
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	// Joystick buttons
	HID_RI_USAGE_PAGE(8, 0x09),			// Button usage page
	HID_RI_USAGE_MINIMUM(8, 0x01),		// Minimum is button 1
	HID_RI_USAGE_MAXIMUM(8, 0x07),		// Maximum is button 7
	HID_RI_LOGICAL_MINIMUM(8, 0x00),	// Minimum value is 0 (off)
	HID_RI_LOGICAL_MAXIMUM(8, 0x01),	// Maximum value is 1 (on)
	HID_RI_REPORT_SIZE(8, 0x01),		// 1 bit per button
	HID_RI_REPORT_COUNT(8, 0x08),		// 8 bits of data are required
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	HID_RI_END_COLLECTION(0),
	
	HID_RI_END_COLLECTION(0)
};

// Joystick 1 report
const USB_Descriptor_HIDReport_Datatype_t PROGMEM JoystickReport1[] =
{
	HID_RI_USAGE_PAGE(8, 0x01),		// Generic Desktop
	HID_RI_USAGE(8, 0x04),			// Joystick
	HID_RI_COLLECTION(8, 0x01),		// Application

	HID_RI_COLLECTION(8, 0x00),		// Physical
	
	// X, Y and throttle
	HID_RI_USAGE(8, 0x30), // Usage X
	HID_RI_USAGE(8, 0x31), // Usage Y
	HID_RI_LOGICAL_MINIMUM(8, -127),
	HID_RI_LOGICAL_MAXIMUM(8, 127),
	HID_RI_PHYSICAL_MINIMUM(8, -1),
	HID_RI_PHYSICAL_MAXIMUM(8, 1),
	HID_RI_REPORT_COUNT(8, 0x02),  // 2 usage reports x and y
	HID_RI_REPORT_SIZE(8, 0x08),
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	// Joystick buttons
	HID_RI_USAGE_PAGE(8, 0x09),			// Button usage page
	HID_RI_USAGE_MINIMUM(8, 0x01),		// Minimum is button 1
	HID_RI_USAGE_MAXIMUM(8, 0x04),		// Maximum is button 4
	HID_RI_LOGICAL_MINIMUM(8, 0x00),	// Minimum value is 0 (off)
	HID_RI_LOGICAL_MAXIMUM(8, 0x01),	// Maximum value is 1 (on)
	HID_RI_REPORT_SIZE(8, 0x01),		// 1 bit per button
	HID_RI_REPORT_COUNT(8, 0x08),		// 8 bits of data are required
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	HID_RI_END_COLLECTION(0),
	
	HID_RI_END_COLLECTION(0)
};

// Joystick 2 report
const USB_Descriptor_HIDReport_Datatype_t PROGMEM JoystickReport2[] =
{
	HID_RI_USAGE_PAGE(8, 0x01),		// Generic Desktop
	HID_RI_USAGE(8, 0x04),			// Joystick
	HID_RI_COLLECTION(8, 0x01),		// Application

	HID_RI_COLLECTION(8, 0x00),		// Physical
	
	// X, Y and throttle
	HID_RI_USAGE(8, 0x30), // Usage X
	HID_RI_USAGE(8, 0x31), // Usage Y
	HID_RI_LOGICAL_MINIMUM(8, -127),
	HID_RI_LOGICAL_MAXIMUM(8, 127),
	HID_RI_PHYSICAL_MINIMUM(8, -1),
	HID_RI_PHYSICAL_MAXIMUM(8, 1),
	HID_RI_REPORT_COUNT(8, 0x02),  // 2 usage reports x and y
	HID_RI_REPORT_SIZE(8, 0x08),
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	// Joystick buttons
	HID_RI_USAGE_PAGE(8, 0x09),			// Button usage page
	HID_RI_USAGE_MINIMUM(8, 0x01),		// Minimum is button 1
	HID_RI_USAGE_MAXIMUM(8, 0x04),		// Maximum is button 4
	HID_RI_LOGICAL_MINIMUM(8, 0x00),	// Minimum value is 0 (off)
	HID_RI_LOGICAL_MAXIMUM(8, 0x01),	// Maximum value is 1 (on)
	HID_RI_REPORT_SIZE(8, 0x01),		// 1 bit per button
	HID_RI_REPORT_COUNT(8, 0x08),		// 8 bits of data are required
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	HID_RI_END_COLLECTION(0),
	
	HID_RI_END_COLLECTION(0)
};

// Joystick 3 report
const USB_Descriptor_HIDReport_Datatype_t PROGMEM JoystickReport3[] =
{
	HID_RI_USAGE_PAGE(8, 0x01),		// Generic Desktop
	HID_RI_USAGE(8, 0x04),			// Joystick
	HID_RI_COLLECTION(8, 0x01),		// Application

	HID_RI_COLLECTION(8, 0x00),		// Physical
	
	// X, Y and throttle
	HID_RI_USAGE(8, 0x30), // Usage X
	HID_RI_USAGE(8, 0x31), // Usage Y
	HID_RI_LOGICAL_MINIMUM(8, -127),
	HID_RI_LOGICAL_MAXIMUM(8, 127),
	HID_RI_PHYSICAL_MINIMUM(8, -1),
	HID_RI_PHYSICAL_MAXIMUM(8, 1),
	HID_RI_REPORT_COUNT(8, 0x02),  // 2 usage reports x and y
	HID_RI_REPORT_SIZE(8, 0x08),
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	// Joystick buttons
	HID_RI_USAGE_PAGE(8, 0x09),			// Button usage page
	HID_RI_USAGE_MINIMUM(8, 0x01),		// Minimum is button 1
	HID_RI_USAGE_MAXIMUM(8, 0x04),		// Maximum is button 4
	HID_RI_LOGICAL_MINIMUM(8, 0x00),	// Minimum value is 0 (off)
	HID_RI_LOGICAL_MAXIMUM(8, 0x01),	// Maximum value is 1 (on)
	HID_RI_REPORT_SIZE(8, 0x01),		// 1 bit per button
	HID_RI_REPORT_COUNT(8, 0x08),		// 8 bits of data are required
	HID_RI_INPUT(8, HID_IOF_DATA | HID_IOF_VARIABLE | HID_IOF_ABSOLUTE),
	
	HID_RI_END_COLLECTION(0),
	
	HID_RI_END_COLLECTION(0)
};

// Device descriptor structure
const USB_Descriptor_Device_t PROGMEM DeviceDescriptor =
{
	.Header                 = {.Size = sizeof(USB_Descriptor_Device_t), .Type = DTYPE_Device},

	.USBSpecification       = VERSION_BCD(01.10),
	.Class                  = USB_CSCP_NoDeviceClass,
	.SubClass               = USB_CSCP_NoDeviceSubclass,
	.Protocol               = USB_CSCP_NoDeviceProtocol,

	.Endpoint0Size          = FIXED_CONTROL_ENDPOINT_SIZE,

	.VendorID               = 0x03EB,
	.ProductID              = 0xA047,
	.ReleaseNumber          = VERSION_BCD(00.01),

	.ManufacturerStrIndex   = 0x01,
	.ProductStrIndex        = 0x02,
	.SerialNumStrIndex      = NO_DESCRIPTOR,

	.NumberOfConfigurations = FIXED_NUM_CONFIGURATIONS
};

// Configuration descriptor structure
const USB_Descriptor_Configuration_t PROGMEM ConfigurationDescriptor =
{
	.Config =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Configuration_Header_t), .Type = DTYPE_Configuration},

			.TotalConfigurationSize = sizeof(USB_Descriptor_Configuration_t),
			.TotalInterfaces        = 4,	// 4 Joystick interfaces

			.ConfigurationNumber    = 1,
			.ConfigurationStrIndex  = NO_DESCRIPTOR,

			.ConfigAttributes       = (USB_CONFIG_ATTR_RESERVED | USB_CONFIG_ATTR_SELFPOWERED),

			.MaxPowerConsumption    = USB_CONFIG_POWER_MA(100)
		},

	// Joystick HID interface (HID0)
	.HID0_Interface =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Interface_t), .Type = DTYPE_Interface},

			.InterfaceNumber        = 0x00, // Interface 0 is joystick 0
			.AlternateSetting       = 0x00,

			.TotalEndpoints         = 1,

			.Class                  = HID_CSCP_HIDClass,
			.SubClass               = HID_CSCP_NonBootSubclass,
			.Protocol               = HID_CSCP_NonBootProtocol,

			.InterfaceStrIndex      = NO_DESCRIPTOR
		},

	.HID0_JoystickHID =
		{
			.Header                 = {.Size = sizeof(USB_HID_Descriptor_HID_t), .Type = HID_DTYPE_HID},

			.HIDSpec                = VERSION_BCD(01.11),
			.CountryCode            = 0x00,
			.TotalReportDescriptors = 1,
			.HIDReportType          = HID_DTYPE_Report,
			.HIDReportLength        = sizeof(JoystickReport0)
		},

	.HID0_ReportINEndpoint =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Endpoint_t), .Type = DTYPE_Endpoint},

			.EndpointAddress        = JOYSTICK0_EPADDR,
			.Attributes             = (EP_TYPE_INTERRUPT | ENDPOINT_ATTR_NO_SYNC | ENDPOINT_USAGE_DATA),
			.EndpointSize           = JOYSTICK_EPSIZE,
			.PollingIntervalMS      = 0x05
		},
		
	// Joystick HID interface (HID1)
	.HID1_Interface =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Interface_t), .Type = DTYPE_Interface},

			.InterfaceNumber        = 0x01, // Interface 1 is joystick 1
			.AlternateSetting       = 0x00,

			.TotalEndpoints         = 1,

			.Class                  = HID_CSCP_HIDClass,
			.SubClass               = HID_CSCP_NonBootSubclass,
			.Protocol               = HID_CSCP_NonBootProtocol,

			.InterfaceStrIndex      = NO_DESCRIPTOR
		},

	.HID1_JoystickHID =
		{
		.Header                 = {.Size = sizeof(USB_HID_Descriptor_HID_t), .Type = HID_DTYPE_HID},

		.HIDSpec                = VERSION_BCD(01.11),
		.CountryCode            = 0x00,
		.TotalReportDescriptors = 1,
		.HIDReportType          = HID_DTYPE_Report,
		.HIDReportLength        = sizeof(JoystickReport1)
		},

	.HID1_ReportINEndpoint =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Endpoint_t), .Type = DTYPE_Endpoint},

			.EndpointAddress        = JOYSTICK1_EPADDR,
			.Attributes             = (EP_TYPE_INTERRUPT | ENDPOINT_ATTR_NO_SYNC | ENDPOINT_USAGE_DATA),
			.EndpointSize           = JOYSTICK_EPSIZE,
			.PollingIntervalMS      = 0x05
		},
		
	// Joystick HID interface (HID2)
	.HID2_Interface =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Interface_t), .Type = DTYPE_Interface},

			.InterfaceNumber        = 0x02, // Interface 2 is joystick 2
			.AlternateSetting       = 0x00,

			.TotalEndpoints         = 1,

			.Class                  = HID_CSCP_HIDClass,
			.SubClass               = HID_CSCP_NonBootSubclass,
			.Protocol               = HID_CSCP_NonBootProtocol,

			.InterfaceStrIndex      = NO_DESCRIPTOR
		},

	.HID2_JoystickHID =
		{
		.Header                 = {.Size = sizeof(USB_HID_Descriptor_HID_t), .Type = HID_DTYPE_HID},

		.HIDSpec                = VERSION_BCD(01.11),
		.CountryCode            = 0x00,
		.TotalReportDescriptors = 1,
		.HIDReportType          = HID_DTYPE_Report,
		.HIDReportLength        = sizeof(JoystickReport1)
		},

	.HID2_ReportINEndpoint =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Endpoint_t), .Type = DTYPE_Endpoint},

			.EndpointAddress        = JOYSTICK2_EPADDR,
			.Attributes             = (EP_TYPE_INTERRUPT | ENDPOINT_ATTR_NO_SYNC | ENDPOINT_USAGE_DATA),
			.EndpointSize           = JOYSTICK_EPSIZE,
			.PollingIntervalMS      = 0x05
		},
		
	// Joystick HID interface (HID3)
	.HID3_Interface =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Interface_t), .Type = DTYPE_Interface},

			.InterfaceNumber        = 0x03, // Interface 3 is joystick 3
			.AlternateSetting       = 0x00,

			.TotalEndpoints         = 1,

			.Class                  = HID_CSCP_HIDClass,
			.SubClass               = HID_CSCP_NonBootSubclass,
			.Protocol               = HID_CSCP_NonBootProtocol,

			.InterfaceStrIndex      = NO_DESCRIPTOR
		},

	.HID3_JoystickHID =
		{
		.Header                 = {.Size = sizeof(USB_HID_Descriptor_HID_t), .Type = HID_DTYPE_HID},

		.HIDSpec                = VERSION_BCD(01.11),
		.CountryCode            = 0x00,
		.TotalReportDescriptors = 1,
		.HIDReportType          = HID_DTYPE_Report,
		.HIDReportLength        = sizeof(JoystickReport1)
		},

	.HID3_ReportINEndpoint =
		{
			.Header                 = {.Size = sizeof(USB_Descriptor_Endpoint_t), .Type = DTYPE_Endpoint},

			.EndpointAddress        = JOYSTICK3_EPADDR,
			.Attributes             = (EP_TYPE_INTERRUPT | ENDPOINT_ATTR_NO_SYNC | ENDPOINT_USAGE_DATA),
			.EndpointSize           = JOYSTICK_EPSIZE,
			.PollingIntervalMS      = 0x05
		}
};

// Language descriptor structure
const USB_Descriptor_String_t PROGMEM LanguageString =
{
	.Header                 = {.Size = USB_STRING_LEN(1), .Type = DTYPE_String},

	.UnicodeString          = {LANGUAGE_ID_ENG}
};

// Manufacturer descriptor string
const USB_Descriptor_String_t PROGMEM ManufacturerString =
{
	.Header                 = {.Size = USB_STRING_LEN(10), .Type = DTYPE_String},

	.UnicodeString          = L"Simon Inns"
};

// Product descriptor string
const USB_Descriptor_String_t PROGMEM ProductString =
{
	.Header                 = {.Size = USB_STRING_LEN(23), .Type = DTYPE_String},

	.UnicodeString          = L"Nintendo Four Score USB"
};

// This function is called by the library when in device mode
uint16_t CALLBACK_USB_GetDescriptor(const uint16_t wValue,
                                    const uint8_t wIndex,
                                    const void** const DescriptorAddress)
{
	const uint8_t  DescriptorType   = (wValue >> 8);
	const uint8_t  DescriptorNumber = (wValue & 0xFF);

	const void* Address = NULL;
	uint16_t    Size    = NO_DESCRIPTOR;

	switch (DescriptorType)
	{
		case DTYPE_Device:
			Address = &DeviceDescriptor;
			Size    = sizeof(USB_Descriptor_Device_t);
			break;
		case DTYPE_Configuration:
			Address = &ConfigurationDescriptor;
			Size    = sizeof(USB_Descriptor_Configuration_t);
			break;
		case DTYPE_String:
			switch (DescriptorNumber)
			{
				case 0x00:
					Address = &LanguageString;
					Size    = pgm_read_byte(&LanguageString.Header.Size);
					break;
				case 0x01:
					Address = &ManufacturerString;
					Size    = pgm_read_byte(&ManufacturerString.Header.Size);
					break;
				case 0x02:
					Address = &ProductString;
					Size    = pgm_read_byte(&ProductString.Header.Size);
					break;
			}

			break;
		case DTYPE_HID:
			switch (wIndex)
			{
				case 0x00:
					Address = &ConfigurationDescriptor.HID0_JoystickHID;
					Size    = sizeof(USB_HID_Descriptor_HID_t);
					break;

				case 0x01:
					Address = &ConfigurationDescriptor.HID1_JoystickHID;
					Size    = sizeof(USB_HID_Descriptor_HID_t);
					break;
				
				case 0x02:
					Address = &ConfigurationDescriptor.HID2_JoystickHID;
					Size    = sizeof(USB_HID_Descriptor_HID_t);
					break;

				case 0x03:
					Address = &ConfigurationDescriptor.HID3_JoystickHID;
					Size    = sizeof(USB_HID_Descriptor_HID_t);
					break;
			}
			break;
		case DTYPE_Report:
			switch (wIndex)
			{
				case 0x00:
					Address = &JoystickReport0;
					Size    = sizeof(JoystickReport0);
					break;

				case 0x01:
					Address = &JoystickReport1;
					Size    = sizeof(JoystickReport1);
					break;

				case 0x02:
					Address = &JoystickReport2;
					Size    = sizeof(JoystickReport2);
					break;

				case 0x03:
					Address = &JoystickReport3;
					Size    = sizeof(JoystickReport3);
					break;
				
			}				
			break;
	}

	*DescriptorAddress = Address;
	return Size;
}

