/************************************************************************

	FourScoreUSB.h

	Nintendo Four Score USB Joystick Adapter
	Copyright (C) 2013 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef FOURSCOREUSB_H_
#define FOURSCOREUSB_H_

// Includes
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/power.h>
#include <avr/interrupt.h>
#include <string.h>

#include "Descriptors.h"

#include "LUFA/Drivers/USB/USB.h"

// Hardware definitions

// Joystick port 1
#define JOY1_DATA_DDR		DDRD
#define JOY1_DATA_PORT		PORTD
#define JOY1_DATA_PIN		PIND
#define JOY1_DATA			(1 << 5)

// Joystick port 2
#define JOY2_DATA_DDR		DDRD
#define JOY2_DATA_PORT		PORTD
#define JOY2_DATA_PIN		PIND
#define JOY2_DATA			(1 << 4)

// Joystick port 3
#define JOY3_DATA_DDR		DDRD
#define JOY3_DATA_PORT		PORTD
#define JOY3_DATA_PIN		PIND
#define JOY3_DATA			(1 << 3)

// Joystick port 4
#define JOY4_DATA_DDR		DDRD
#define JOY4_DATA_PORT		PORTD
#define JOY4_DATA_PIN		PIND
#define JOY4_DATA			(1 << 2)

// Joystick clock line (shared by all 4 joystick ports)
#define JOY_CLOCK_DDR		DDRD
#define JOY_CLOCK_PORT		PORTD
#define JOY_CLOCK_PIN		PIND
#define JOY_CLOCK			(1 << 0)

// Joystick latch line (shared by all 4 joystick ports)
#define JOY_LATCH_DDR		DDRD
#define JOY_LATCH_PORT		PORTD
#define JOY_LATCH_PIN		PIND
#define JOY_LATCH			(1 << 1)

// Turbo A button
#define TURBOA_DDR			DDRD
#define TURBOA_PORT			PORTD
#define TURBOA_PIN			PIND
#define TURBOA				(1 << 6)

// Turbo B button
#define TURBOB_DDR			DDRD
#define TURBOB_PORT			PORTD
#define TURBOB_PIN			PIND
#define TURBOB				(1 << 7)

// Players switch
#define PLAYERS_DDR			DDRC
#define PLAYERS_PORT		PORTC
#define PLAYERS_PIN			PINC
#define PLAYERS				(1 << 4)

// Mapping of buttons to the state array
#define MAP_FIREA_BUTTON	0
#define MAP_FIREB_BUTTON	1
#define MAP_SELECT_BUTTON	2
#define MAP_START_BUTTON	3
#define MAP_DOWN_BUTTON		4
#define MAP_UP_BUTTON		5
#define MAP_LEFT_BUTTON		6
#define MAP_RIGHT_BUTTON	7

#define MAP_PLAYERS_BUTTON	8
#define MAP_TURBOA_BUTTON	9
#define MAP_TURBOB_BUTTON	10

// The number of physical buttons
#define NUMBER_OF_BUTTONS		11

// Define the de-bounce tolerance
#define DEBOUNCE_TOLERANCE		10

// Physical button states
#define BUTTON_OFF	0
#define BUTTON_ON	1

// Type definitions:
typedef struct
{
	int8_t  x;			// Current absolute joystick X position, as a signed 8-bit integer
	int8_t  y;			// Current absolute joystick Y position, as a signed 8-bit integer
	uint8_t buttonMask; // Bit mask of the currently pressed joystick buttons
} USB_JoystickReport_Data_t;

// Function prototypes
void SetupHardware(void);
void HID_Task(void);

void EVENT_USB_Device_Connect(void);
void EVENT_USB_Device_Disconnect(void);
void EVENT_USB_Device_ConfigurationChanged(void);
void EVENT_USB_Device_ControlRequest(void);

void readButtonStates(void);
void readJoystickStates(void);
void performDebounce(void);

bool GetNextReport(USB_JoystickReport_Data_t* const reportData, USB_JoystickReport_Data_t* const previousReportData, uint8_t joystickNumber);

#endif /* FOURSCOREUSB_H_ */